<?php
/**
 * Copyright (c) 2009 Brian Armstrong
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 * THE SOFTWARE.
 */

/**
 * Class to hold a GeoCode result for return from a given API
 *
 * @package GeoCoder
 * @author Brian Armstrong <brian@barmstrongconsulting.com>
 * @copyright (C) 2009 Brian Armstrong
 * @link http://barmstrongconsulting.com/
 * @version 1.0
 */
class GeoCode_Result
{
	/**
	 * The driver associated with this result
	 * @var GeoCode_Driver
	 */
	private $_driver = null;

	/**
	 * The attributes array to hold all of our data
	 * @var array
	 */
	protected $attributes = array(
		/**
		 * The accuracy code
		 * @var integer
		 */
		'accuracy' => null,
		/**
		 * The latitude in degrees
		 * @var float
		 */
		'latitude' => null,
		/**
		 * The longitude in degrees
		 * @var float
		 */
		'longitude' => null,
		/**
		 * The query that was sent to the API
		 * @var string
		 */
		'query' => null,
		/**
		 * The processed query, as returned by the API
		 * @var string
		 */
		'clean_query' => null,
		/**
		 * The state name
		 * @var string
		 */
		'state' => null,
		/**
		 * The county name
		 * @var string
		 */
		'county' => null,
		/**
		 * The city name
		 * @var string
		 */
		'city' => null,
		/**
		 * The zip code
		 * NOTE: This needs to be treated as a string. Zip codes outside of the
		 *  United states are not guaranteed to be numeric. Example: Canada
		 * @var string
		 */
		'zip' => null,
		/**
		 * The street address
		 * @var string
		 */
		'street' => null,
		/**
		 * The country
		 * @var string
		 */
		'country' => null,
	);

	/**
	 * Initialize the result object
	 *
	 * @param GeoCode_Driver $driver
	 * @param array $data
	 * @return GeoCode_Result
	 */
	public function __construct(GeoCode_Driver $driver, $data = array())
	{
		$this->_driver = $driver;

		// Assign all the data
		foreach ($data as $key => $value)
		{
			// Pass off to the set method
			$this->__set($key, $value);
		}
	}

	/**
	 * Get the string associated with the accuracy constant
	 *
	 * @return string
	 */
	public function getAccuracyString()
	{
		return $this->_driver->getAccuracyString($this->accuracy);
	}

	/**
	 * Get the latitude in radians
	 *
	 * @return float
	 */
	public function getLatitudeRadians()
	{
		return isset($this->latitude) ? deg2rad((float)$this->latitude) : null;
	}

	/**
	 * Get the longitude in radians
	 *
	 * @return float
	 */
	public function getLongitudeRadians()
	{
		return isset($this->longitude) ? deg2rad((float)$this->longitude) : null;
	}

	/**
	 * Magic get method. If a get<Name> method exists, then it is called.
	 * Otherwise we look in the attributes array for the key.
	 * If nothing is found, an exception is thrown
	 *
	 * @param string $name
	 * @return mixed
	 */
	public function __get($name)
	{
		$method = 'get'.str_replace('_', '', $name);
		if (method_exists($this, $method))
		{
			return call_user_func(array($this, $method));
		}
		elseif (array_key_exists($name, $this->attributes))
		{
			return $this->attributes[$name];
		}
		else
		{
			throw new CException("Property '{$key}' does not exists");
		}
	}

	/**
	 * Magic set method. If a set<Name> method exists, then it is called.
	 * Otherwise we look in the attributes array for the key.
	 * If nothing is found, an exception is thrown
	 *
	 * @param string $name
	 * @param mixed $value
	 */
	public function __set($name, $value)
	{
		$method = 'set'.str_replace('_', '', $name);
		if (method_exists($this, $method))
		{
			call_user_func(array($this, $method), $value);
		}
		elseif (array_key_exists($name, $this->attributes))
		{
			$this->attributes[$name] = $value;
		}
		else
		{
			throw new CException("Property '{$name}' does not exists");
		}
	}

	/**
	 * Magic isset function. Checks if an attribute is set
	 *
	 * @param string $name
	 * @return boolean
	 */
	public function __isset($name)
	{
		$method = 'get'.str_replace('_', '', $name);
		if (method_exists($this, $method))
		{
			$value = call_user_func(array($this, $method));
			return isset($value); // NULL returns FALSE
		}
		else
		{
			return (isset($this->attributes[$name]));
		}
	}

	/**
	 * Magic unset function. Removes a given attribute
	 *
	 * @param string $name
	 */
	public function __unset($name)
	{
		$method = 'set'.str_replace('_', '', $name);
		if (method_exists($this, $method))
		{
			call_user_func(array($this, $method), null);
		}
		else
		{
			// Set to NULL, isset will return false
			$this->attributes[$name] = null;
		}
	}
}
